#include "crossroads.h"

static jmp_buf jmpbuf;
static int timeout;

static void alarmhandler (int sig) {
    alarm(0);
    longjmp (jmpbuf, 1);
}


int backend_connect () {
    int backend_sock;
    struct sockaddr_in servername;

    if ( (backend_sock = socket (PF_INET, SOCK_STREAM, 0)) < 0 ) {
	if (program_stage != stage_retrying) {
	    error ("Service %s: failed to create socket "
		   "for backend communication: %s",
		   activeservice->name, strerror(errno));
	} else {
	    warning ("Service %s: failed to create socket "
		     "for backend communication: %s",
		     activeservice->name, strerror(errno));
	    return (-1);
	}
    }
    
    if (init_sockaddr (&servername,
		       activeservice->backend[current_backend].server,
		       activeservice->backend[current_backend].port)) {
	/* The hostname is unusable.. */
	warning ("Service %s: unknown host %s",
		 activeservice->name,
		 activeservice->backend[current_backend].server);
	mark_activity (0, 0, st_unavailable);
	return (-2);
    }

    signal (SIGALRM, alarmhandler);
    timeout = 0;
    alarm (CONNECT_TIMEOUT);
    
    if (!setjmp (jmpbuf)) {
	/* First time around, try connecting */
	if (connect (backend_sock, (struct sockaddr *) &servername,
		     sizeof(servername)) < 0) {
	    /* This backend is unusable..
	     * Either connect() has failed (connection refused) or it got
	     * interrupted by the SIGALRM
	     */
	    close (backend_sock);
	    alarm(0);
	    mark_activity (0, 0, st_unavailable);	    
	    if (timeout) {
		if (program_stage != stage_retrying)
		    warning ("Service %s: server %s not usable due to timeout",
			     activeservice->name,
			     activeservice->backend[current_backend].server);
	    }  else {
		if (program_stage != stage_retrying)
		    warning ("Service %s: server %s:%d: cannot connect: %s",
			     activeservice->name,
			     activeservice->backend[current_backend].server,
			     activeservice->backend[current_backend].port,
			     strerror(errno));
		mark_activity (0, 0, st_unavailable);
	    }
	    return (-3);
	}
	alarm(0);
    } else {
	/* Got here because of longjmp from the signal handler,
	 * this MUST be a timeout
	 */
	close (backend_sock);
	mark_activity (0, 0, st_unavailable);
	if (program_stage != stage_retrying)
	    warning ("Server %s not usable due to timeout",
		     activeservice->backend[current_backend].server);
	return (-4);
    }

    signal (SIGALRM, SIG_DFL);
    return (backend_sock);
}
