#include "crossroads.h"

static void copy (int src, int dst, int dir, unsigned tot) {
    unsigned ncopied = 0, to_copy, bytes;
    unsigned char *buf;

    while (ncopied < tot) {
	/* Here's how many bytes we should shuttle to and fro.
	 * This is the remaining count, or if the buffer is too small
	 * for that, the buffer size. */
	to_copy = tot - ncopied;
	if (to_copy > TCP_BUFSZ)
	    to_copy = TCP_BUFSZ;

	buf = net_bufread (src, to_copy, &bytes, dir == dir_client_to_server);
	net_write (dst, buf, bytes, dir == dir_client_to_server);
	ncopied += bytes;

	/* msg ("Copied 0x%x (%d) bytes of 0x%x (%d) total",
	     ncopied, ncopied, tot, tot);
	*/
    }
}

unsigned getchunk (int sock, int dest, CopyDirection dir) {
    unsigned ret = 0;
    unsigned char *cp;
    char *hexbuf = 0;
    int in_number = 1;

    /* We match
     *           <hex-chunksize> <any-mush> [\r] \n
     * while copying all to the other socket, and return the
     * chunk size */
    while (1) {
	cp = net_bufread (sock, 1, 0, dir == dir_client_to_server);
	// msg ("Got character %c (%d) while scanning chunk size", *cp, *cp);
	net_write (dest, cp, 1, dir == dir_client_to_server);
	if (*cp == '\n') {
	    sscanf (hexbuf, "%x", &ret);
	    free (hexbuf);
	    // msg ("Got chunk size 0x%x (%d)", ret, ret);
	    return (ret);
	}
	if (in_number) {
	    if (is_hex_digit (*cp))
		hexbuf = xstrcatch (hexbuf, *cp);
	    else
		in_number = 0;
	}
    }
}

void http_copy (HttpHeader *h, int src, int dst, CopyDirection dir) {
    unsigned char const *mode;
    unsigned nbytes;
    
    msg ("Service %s: copying HTTP body from %s",
	 activeservice->name,
	 dir == dir_client_to_server ?
	   "client to server" : "server to client");

    /* Check in what 'mode' we are.
     * We know 3 modes:
     * - Transfer-Encoding is chunked: we copy chunks
     * - Content-Length is supplied: we copy that # of bytes
     * - None of the above: don't do anything, http_serve() will go
     *   into copy-thru mode
     */
  
    if ( (mode = http_header_val (h, "transfer-encoding")) && *mode ) {
	if (strncasecmp ( (char const *) mode, "chunked", 7))
	    error ("Service %s: can't handle transfer-encoding '%s'",
		   activeservice->name, mode);
	msg ("Service %s: copying chunks from %s",
	     activeservice->name, dir == dir_client_to_server ?
	         "client to server" : "server to client");
	while (1) {
	    /* See how large the chunk will be. */
	    nbytes = getchunk (src, dst, dir);
	    msg ("Service %s: Next chunk is 0x%x (%d) bytes",
		 activeservice->name, nbytes, nbytes);
	    /* Copy the chunk. Add 2 bytes for the \r\n that follows. */
	    copy (src, dst, dir, nbytes + 2);
	    /* Last chunksize of 0 signals the end. */
	    if (!nbytes)
		break;
	}
    } else if ( (mode = http_header_val (h, "content-length")) &&
		mode ) {
	nbytes = atoi ( (char const *) mode );
	msg ("Service %s: copying %u bytes of content from %s",
	     activeservice->name, nbytes, 
	     dir == dir_client_to_server ?
	         "client to server" : "server to client");
	copy (src, dst, dir, nbytes);
    } else
	msg ("Service %s: no body to send", activeservice->name);
}
