#include "crossroads.h"

int http_serversocket (HttpHeader *m, int *is_continuation) {
    int i, sock;

    msg ("Service %s: searching for back end for HTTP request.",
	 activeservice->name);

    /* Try to find a sticky cookie in the request. */
    for (i = 0; i < activeservice->nbackend; i++) {
	if (servicereport->backendstate[i].avail != st_available)
	    continue;
	if (activeservice->backend[i].maxconnections > 0 &&
	    activeservice->backend[i].maxconnections <=
	    servicereport->backendstate[i].nclients)
	    continue;
	if (http_header_hascookie (m,
				   activeservice->backend[i].stickycookie)) {
	    msg ("Service %s: HTTP backend %d selected due to cookie '%s'",
		 activeservice->name,
		 i, 
		 activeservice->backend[i].stickycookie);

	    /* Got the target back end. Try to connect to it.
	     * If it doesn't succeed, then we'll do a failover to
	     * any other back end. */
	    current_backend = i;
	    if ( (sock = backend_connect ()) >= 0 ) {
		*is_continuation = 1;
		return (sock);
	    }
	}
    }

    /* We got here because this is the first time 'round or because
     * a previously established back end went dead in the middle of a
     * session. So: Loop 'till we find a back end, or until we fail. */
    while (1) {
	/* No back end? Nogo. */
	if (! backend_available()) {
	    msg ("Service %s: "
		 "out of back ends while scanning for HTTP back end",
		 activeservice->name);
	    return (-1);
	}

	/* Try one. */
	choose_backend();
	if (current_backend < 0) {
	    warning ("No back end for request.");
	    return (-2);
	}

	msg ("Service %s: trying back end %d for new HTTP connection",
	     activeservice->name, current_backend);
	if ( (sock = backend_connect ()) >= 0 ) {
	    servicereport->backendstate[current_backend].sessions++;
	    *is_continuation = 0;
	    return (sock);
	}
    }
}
