    
#include "crossroads.h"

int make_socket (int port, char const *ipaddr) {
    int sock, val = 1;
    struct sockaddr_in name;

    /* Create the socket. */
    if ( (sock = socket (PF_INET, SOCK_STREAM, 0)) < 0) {       
	warning ("Service %s: cannot create network socket: %s",
		 activeservice->name, strerror(errno));
	return (-1);
    }
    if (setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))) {
	warning ("Service %s: cannot set socket options: %s",
		 activeservice->name, strerror(errno));
	close (sock);
	return (-2);
    }

    /* Give the socket a name. */
    name.sin_family = AF_INET;
    name.sin_port = htons (port);

    /* Propagate 'bindto' if given. */
    if (strcasecmp (ipaddr, "any")) {
	msg ("Service %s: creating socket for IP address '%s'",
	     activeservice->name, ipaddr);
	if ( (name.sin_addr.s_addr = inet_addr (ipaddr)) == INADDR_NONE ) {
	    close (sock);
	    error ("Service %s: cannot convert address '%s' to network bytes",
		   activeservice->name, ipaddr);
	}
    } else {
	msg ("Service %s: creating socket for all interfaces",
	     activeservice->name);
	name.sin_addr.s_addr = htonl (INADDR_ANY);
    }

    /* Finallly, bind the socket. */
    if (bind (sock, (struct sockaddr *) &name, sizeof (name)) < 0) {
	warning ("Cannot bind socket to port %d: %s",
		 port, strerror(errno));
	close (sock);
	return (-3);
    }

    return (sock);
}
