#include "crossroads.h"

void mark_activity (unsigned long long nbytes, double nsec,
		    Backendavail newstate) {
    unsigned long long llval;
    double dlval;
    unsigned multiplier;
    int i;

    /* If we had a signal somewhere, don't update the values.
     * If we don't have a back end yet, don't update the values. */
    if 	(interrupted || current_backend == -1)
	return;

    /* Update values. */
    lock_reporter();

    /* Handle the decays of all other back ends */
    for (i = 0; i < activeservice->nbackend; i++) {
	if (i != current_backend && activeservice->backend[i].decay) {
	    if (activeservice->dispatchover) {
		servicereport->backendstate[i].avg_nbytes *=
		    (100 - activeservice->backend[i].decay);
		servicereport->backendstate[i].avg_nbytes /= 100;
		servicereport->backendstate[i].avg_nsec *=
		    (100 - activeservice->backend[i].decay);
		servicereport->backendstate[i].avg_nsec /= 100;
	    } else {
		servicereport->backendstate[i].nbytes *=
		    (100 - activeservice->backend[i].decay);
		servicereport->backendstate[i].nbytes /= 100;
		servicereport->backendstate[i].nsec *=
		    (100 - activeservice->backend[i].decay);
		servicereport->backendstate[i].nsec /= 100;
	    }
	}
    }

    /* Update average seconds and bytes. */
    if (activeservice->dispatchover) {
	multiplier = activeservice->dispatchover - 1;
	if (servicereport->backendstate[current_backend].totuses < multiplier)
	    multiplier = servicereport->backendstate[current_backend].totuses;

	dlval = servicereport->backendstate[current_backend].avg_nsec;
	dlval *= multiplier;
	dlval += nsec;
	dlval /= (multiplier + 1);
	servicereport->backendstate[current_backend].avg_nsec = dlval;

	llval = servicereport->backendstate[current_backend].avg_nbytes;
	llval *= multiplier;
	llval += nbytes;
	llval /= (multiplier + 1);
	servicereport->backendstate[current_backend].avg_nbytes = llval;
    }

    /* Update total secs / bytes. */
    servicereport->backendstate[current_backend].nsec += nsec;
    servicereport->backendstate[current_backend].nbytes += nbytes;
    if (newstate == st_unavailable)
	servicereport->backendstate[current_backend].failures++;

    /* Set the state, unless it's already marked for wakeup or brought down.
     * Do this only for 'final' states, not for intermediate. */
    if (newstate != st_intermediate) {
	if (servicereport->backendstate[current_backend].avail != st_waking &&
	    servicereport->backendstate[current_backend].avail != st_down)
	    servicereport->backendstate[current_backend].avail = newstate;
	servicereport->backendstate[current_backend].totuses++;
    }

    /* Don't update servicereport beyond this point! */
    unlock_reporter();

    /* Give some feedback (though not during intermediate updates) */      
    if (newstate != st_intermediate && program_stage != stage_retrying)
	msg ("Service %s: updated stats for backend %d (%s): "
	     "hits=%lu, "
	     "fails=%lu, secs=%g, avgsecs=%g, "
	     "bytes=%llu, avgbytes=%lu, state=%s",
	     activeservice->name,
	     current_backend, activeservice->backend[current_backend].name,
	     servicereport->backendstate[current_backend].totuses,
	     servicereport->backendstate[current_backend].failures,
	     servicereport->backendstate[current_backend].nsec,
	     servicereport->backendstate[current_backend].avg_nsec,
	     servicereport->backendstate[current_backend].nbytes,
	     servicereport->backendstate[current_backend].avg_nbytes,
	     state_to_string
	         (servicereport->backendstate[current_backend].avail));

    /* At the end of a request, check the onsuccess/onfailure handlers.
     * We only do this for the 'final' states. */
    if (newstate == st_available)
	sysrun (activeservice->backend[current_backend].onsuccess);
    else if (newstate == st_unavailable)
	sysrun (activeservice->backend[current_backend].onfailure);
}
