#include "crossroads.h"

unsigned net_copy (int cl, int sr, unsigned max, unsigned char *buf) {
    fd_set readset, exset;
    struct timeval tv, *tvp, tv1, tv2;
    int nfd;
    unsigned nread, nwritten, totwritten;
    CopyDirection dir;
    double microsec;

    /* Prepare select() sets */
    FD_ZERO (&readset);
    FD_ZERO (&exset);

    FD_SET (cl, &readset);
    FD_SET (cl, &exset);
    FD_SET (sr, &readset);
    FD_SET (sr, &exset);

    /* Prepare timout state */
    if (activeservice->connectiontimeout) {
	tv.tv_sec = activeservice->connectiontimeout;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;

    /* Wait for a socket to become readable */
    gettimeofday (&tv1, 0);
    nfd = select (FD_SETSIZE, &readset, 0, &exset, tvp);
    if (nfd < 1) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, st_available);
	error ("Service %s: network copy stopped after timeout",
	       activeservice->name);
    }

    /* Check for exceptions. */
    if (FD_ISSET (cl, &exset)) {
	decr_client_count();
	log_activity_end();
	error ("Service %s: client exception", activeservice->name);
    }
    if (FD_ISSET (sr, &exset)) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, st_unavailable);
	error ("Service %s: server exception", activeservice->name);
    }

    /* Do the read. */
    if (FD_ISSET (sr, &readset))
	dir = dir_server_to_client;
    else
	dir = dir_client_to_server;

    nread = read (dir == dir_client_to_server ? cl : sr, buf, max);
    if (nread < 1) {
	if (nread < 0) {
	    if (dir == dir_server_to_client)
		mark_activity (0, 0, st_unavailable);
	    error ("Service %s: read error when getting data from %s",
		   activeservice->name,
		   dir == dir_client_to_server ? "client" : "server");
	} else {
	    msg ("Service %s: %s signals end of data",
		 activeservice->name,
		 dir == dir_client_to_server ? "client" : "server");
	    exit (0);
	}
    }

    /* Update thruput / traffic logs. */
    trafficlog (buf, nread, dir);
    thruputlog (buf, nread, dir);

    /* Write to dest. */
    totwritten = 0;
    while (totwritten < nread) {
	nwritten = write (dir == dir_client_to_server ? sr : cl,
			  buf + totwritten,
			  nread - totwritten);
	if (nwritten < 1) {
	    if (dir == dir_client_to_server)
		mark_activity (0, 0, st_unavailable);
	    decr_client_count();
	    log_activity_end();
	    error ("Service %s: write error when sending data to %s",
		   dir == dir_client_to_server ? "server" : "client");
	}
	totwritten += nwritten;
    }

    gettimeofday (&tv2, 0);
    microsec = ( (tv2.tv_sec * 1000000 + tv2.tv_usec) -
                 (tv1.tv_sec * 1000000 + tv1.tv_usec) );

    mark_activity (totwritten, microsec / 1000000, st_intermediate);

    return (nread);
}
