#include "crossroads.h"

unsigned net_read (int sock, unsigned max, unsigned char *buf, int is_client) {
    fd_set readset, exset;
    struct timeval tv, *tvp, tv1, tv2;
    int nfd;
    unsigned nread;
    CopyDirection dir;
    double microsec;
    char *desc;

    desc = is_client ? "client" : "server";
    dir  = is_client ? dir_client_to_server : dir_server_to_client;

    msg ("Service %s: fetching atmost %u bytes from %s",
	 activeservice->name, max, desc);

    /* Prepare select() sets */
    FD_ZERO (&readset);
    FD_ZERO (&exset);
    FD_SET (sock, &readset);
    FD_SET (sock, &exset);

    /* Prepare timout state */
    if (activeservice->connectiontimeout) {
	tv.tv_sec = activeservice->connectiontimeout;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;

    /* Wait for the socket to become readable */
    gettimeofday (&tv1, 0);
    nfd = select (FD_SETSIZE, &readset, 0, &exset, tvp);
    if (nfd < 1) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, is_client ? st_available : st_unavailable);
	error ("Service %s: read from %s timed out",
	       activeservice->name, desc);
    }

    /* Check for exceptions. */
    if (FD_ISSET (sock, &exset)) {
	decr_client_count();
	log_activity_end();
	mark_activity (0, 0, is_client ? st_available : st_unavailable);
	error ("Service %s: %s exception", desc, activeservice->name);
    }

    /* Do the read. */
    nread = read (sock, buf, max - 1);
    if (nread < 1) {
	if (nread < 0) {
	    mark_activity (0, 0, is_client ? st_available : st_unavailable);
	    error ("Service %s: read error when getting data from %s",
		   activeservice->name, desc);
	} else {
	    msg ("Service %s: %s signals end of data",
		 activeservice->name, desc);
	    exit (0);
	}
    }

    /* Update thruput / traffic logs. */
    trafficlog (buf, nread, dir);
    thruputlog (buf, nread, dir);
    gettimeofday (&tv2, 0);
    microsec = ( (tv2.tv_sec * 1000000 + tv2.tv_usec) -
                 (tv1.tv_sec * 1000000 + tv1.tv_usec) );

    mark_activity (nread, microsec / 1000000, st_intermediate);

    return (nread);
}
