#include "crossroads.h"

/* Define DUMPFILE if you want Crossroads to dump the contents of what's
 * written to the client or to the server to
 * /tmp/crossroads-sentto{client,server}.dump. This is of course for deep
 * mode debugging only.
 */	
#define DUMPFILE

int net_write (int sock, unsigned char const *buf, unsigned buflen,
	       int is_client) {
    int ret, nfd;
    struct timeval tv, tv1, tv2, *tvp;
    double microsec;
    fd_set fdset, exset;

    #ifdef DUMPFILE
    FILE *f;
    char *fname;

    if (is_client)
	fname =  "/tmp/crossroads-senttobackend.dump";
    else
	fname =  "/tmp/crossroads-senttoclient.dump";
    if (! (f = fopen (fname, "a")))
	f = fopen (fname, "w");
    if (f) {
	fprintf (f, "[0x%x %d bytes] ", buflen, buflen);
	fwrite (buf, 1, buflen, f);
	fclose (f);
    }
    #endif

    /* Start timer */
    gettimeofday (&tv1, 0);

    /* Prepare the write. */
    FD_ZERO (&fdset);
    FD_ZERO (&exset);
    FD_SET (sock, &fdset);
    FD_SET (sock, &exset);
    if (activeservice->connectiontimeout) {
	tv.tv_sec  = activeservice->connectiontimeout;
	tv.tv_usec = 0;
	tvp = &tv;
    } else
	tvp = 0;
    if ( (nfd = select (FD_SETSIZE, 0, &fdset, &exset, tvp)) < 1) {
	if (!nfd)
	    warning ("Service %s: Timout while writing %s on socket %d",
		     activeservice->name,
		     is_client ? "client" : "server", sock);
	else
	    warning ("Service %s: Failed to wait for network of %s "
		     "on socket %d",
		     activeservice->name,
		     is_client ? "client" : "server", sock);
	
	decr_client_count();
	log_activity_end();
	if (!is_client)
	    mark_activity (0, 0, st_unavailable);
	exit (0);
    }

    if (FD_ISSET (sock, &exset)) {
	decr_client_count();
	log_activity_end();
	error ("Service %s: exception on %s network",
	       is_client ? "client" : "server");
    }

    /* We can write! */
    ret = write (sock, buf, buflen);

    /* Check for finished / errors */
    if (ret < 1) {
	if (ret < 0 && !is_client)
	    mark_activity (0, 0, st_unavailable);
	decr_client_count();
	log_activity_end ();
	if (ret < 0)
	    error ("Service %s: write error when copying to %s",
		   activeservice->name, is_client ? "client" : "server");
    }

    /* Update activity. */
    gettimeofday (&tv2, 0);
    microsec = ( (tv2.tv_sec * 1000000 + tv2.tv_usec) -
		 (tv1.tv_sec * 1000000 + tv1.tv_usec) );
    mark_activity (ret, microsec / 1000000, st_intermediate);

    /* Signal caller how many bytes were written. */
    return (ret);
}
	
	
