#include "crossroads.h"

void runservice () {
    int pid;

    msg ("Service %s (on port %d): STARTING",
	 activeservice->name, activeservice->port);
    
    /* Allocate the shared mem segment. */
    alloc_reporter(activeservice, 1);
    
    /* Go into the background. */
    if ( (pid = fork()) < 0 ) {
	/* Fork failed */
	error ("Service %s: fork failure: %s",
	       activeservice->name, strerror(errno));
    } else if (pid) {
	/* Parent branch */
	msg ("Service %s: detached as PID %d",
	     activeservice->name, pid);
	return;
    } else {
	/* Child branch */
	set_program_title ("service %s listening", activeservice->name);
	close (0);
	close (1);
	close (2);
	daemonized++;
	if ( (open ("/dev/null", O_RDONLY) < 0) ||
	     (open ("/dev/null", O_WRONLY) < 0) ||
	     (open ("/dev/null", O_WRONLY) < 0) )
	    error ("Service %s: "
		   "failed to reopen stdin/out/err on /dev/null",
		   activeservice->name);
	if (setsid() < 0)
	    error ("Service %s: failed to become seesion leader",
		   activeservice->name);
	    
	/* Store our pid, so that 'crossroads stop' may kill us. */
	lock_reporter();
	servicereport->pid = getpid();
	unlock_reporter();

	/* Promote verbosity. */
	flag_verbose = activeservice->verbosity;
	
	/* We've forked for the first time */
	program_stage = stage_waiting;
    }

    /* In 'forever' mode, we create a server-side socket and ask tcpserve()
     * to service it. tcpserve() will return to us if there are no back
     * ends to service the request. In that case, we close the listening
     * socket (so that new clients get denied), take a nap, and redo.
     * Of course we only start listening if we have back ends at all...
     */
    while (1) {
	if (backend_available()) {
	    /* Create the socket, bind to port. */
	    if ( (listen_sock = make_socket (activeservice->port,
					     activeservice->bind)) < 0 ) {
		warning ("Service %s: "
			 "listening socket creation failed.. will retry",
			 activeservice->name);

		/* We wait here for some time, to let the system regain
		 * equilibrium. Something's really afoot, so let's not retry
		 * right away. */
		sleep (SLEEP_TIME);
		
		continue;
	    }
	    msg ("Service %s: server-side network socket: %d",
		 activeservice->name, listen_sock);
	    
	    /* Start serving the port! */
	    tcpserve (listen_sock);
	    
	    /* tcpserve() returned -- must be because this service is out
	     * of back ends */
	    if (shutdown (listen_sock, SHUT_RDWR))
		error ("Service %s: "
		       "failed to shut down server-side socket %d: %s",
		       activeservice->name, listen_sock, strerror(errno));
	    if (close (listen_sock))
		error ("Service %s: failed to close server-side socket %d: %s",
		       activeservice->name, listen_sock, strerror(errno));
	    
	}
	
	msg ("Service %s: taking a nap...", activeservice->name);
	sleep (SLEEP_TIME);
    }
}
