#include "crossroads.h"

static char *timestr (double nsec) {
    static char *buf;
    char *tmp;
    
    free (buf);
    buf = 0;

    if (nsec > 3600) {
	buf = str_printf ("%uh", (unsigned) (nsec / 3600));
	nsec = fmod (nsec, 3600);
    }
    if (nsec > 60) {
	tmp = str_printf ("%um", (unsigned) (nsec / 60));
	nsec = fmod (nsec, 60);
	buf = xstrcat (buf, tmp);
	free (tmp);
    }
    tmp = str_printf ("%.2f", nsec);
    buf = xstrcat (buf, tmp);
    free (tmp);

    return (buf);
}

static char *bytestr (unsigned long long nbytes) {
    static char *buf;

    free (buf);
    buf = 0;

    if (nbytes > 1024*1024*1024)
	buf = str_printf ("%.2fGb", (double) nbytes / (1024*1024*1024));
    if (nbytes > 1024*1024)
	buf = str_printf ("%.2fMb", (double) nbytes / (1024*1024));
    else if (nbytes > 1024)
	buf = str_printf ("%.2fKb", (double) nbytes / 1024);
    else
	buf = str_printf ("%llub", nbytes);

    return (buf);
}

void show_status (int ac, char **av) {
    int i, j;

    for (i = 0; i < nservice; i++) {
	if (i)
	    putchar ('\n');
	alloc_reporter (service + i, 0);
	msg ("Reporting service %s (pid = %d)",
	     service[i].name, servicereport->pid);
	printf ("Service      : %s, %d connections, last backend %d\n",
		service[i].name, servicereport->nclients,
		servicereport->last_backend);
	for (j = 0; j < service[i].nbackend; j++) {
            printf ("  Backend %2d : %s is %s, %u connections\n"
                    "    Stats    : %lu failed in %lu connections,",
		    j,
		    service[i].backend[j].name,
		    state_to_string (servicereport->backendstate[j].avail),
		    servicereport->backendstate[j].nclients,
		    servicereport->backendstate[j].failures,
		    servicereport->backendstate[j].totuses);
	    if (service[i].type == type_http)
		printf (" %lu sessions,",
			servicereport->backendstate[j].sessions);
            printf (" usage %s, %s",
		    timestr (servicereport->backendstate[j].nsec),
		    bytestr (servicereport->backendstate[j].nbytes));
	    if (service[i].dispatchover)
		printf (" avg %s, %s",
			timestr(servicereport->backendstate[j].avg_nsec),
			bytestr(servicereport->backendstate[j].avg_nbytes));
	    putchar ('\n');
	}
	
	if (shmdt (servicereport) < 0)
	    error ("Failure releasing reporter memory: %s",
		   strerror(errno));
    }
}
