#include "crossroads.h"

void thruputlog (unsigned char const *buf, int len, CopyDirection dir) {
    struct timeval tv;
    static double d_start = 0;
    double d_now;
    int i;
    FILE *f;

    if (current_backend < 0 ||
	! activeservice->backend[current_backend].thruputfile)
	return;

    /* Initialize timer if necessary. Get current time. */
    if (! d_start) {
	if (gettimeofday (&tv, 0))
	    error ("Failed to get the time of day: %s\n",
		   strerror(errno));
	d_start = tv.tv_sec * 1000000 + tv.tv_usec;
    }
    if (gettimeofday (&tv, 0))
	error ("Failed to get the time of day: %s\n", strerror(errno));
    d_now = tv.tv_sec * 1000000 + tv.tv_usec;
	
    /* Get a handle on the reporting log. */
    if ( (! (f = fopen (activeservice->backend[current_backend].thruputfile,
			"a"))) &&
	 (! (f = fopen (activeservice->backend[current_backend].thruputfile,
			"w"))) ) {
	warning ("Cannot write %s: %s",
		 activeservice->backend[current_backend].thruputfile,
		 strerror(errno));
	return;
    }

#if defined HAVE_FLOCK    
    flock (fileno(f), LOCK_EX);
#elif defined HAVE_LOCKF
    lockf (fileno(f), F_LOCK, 0);
#endif    
    
    /* Report the activity. */
    fprintf (f, "%7.7d %15f %c ",
	     getpid(),
	     (d_now - d_start) / 1000000,
	     dir == dir_client_to_server ? 'C' : 'B');
    for (i = 0; i < 100 && i < len; i++)
	fputc (isprint (buf[i]) ? buf[i] : '.', f);
    fputc ('\n', f);
    
#if defined HAVE_FLOCK    
    flock (fileno(f), LOCK_UN);
#elif defined HAVE_LOCKF
    lockf (fileno(f), F_ULOCK, 0);
#endif

    fclose (f);
}
