#include "crossroads.h"

void wakeup_handler () {
    int sock, i;
    
    /* Set the stage and any signals that we want. */
    program_stage = stage_retrying;
    for (i = 0; relevant_sigs[i]; i++)
	signal (relevant_sigs[i], interrupt);

    /* Forever, until a signal kills us, or Sol explodes, causing
     * massive electromagnetic pulses that fry my CPU.
     */
    while (1) {
	/* Wait for the alarm to go off. */
	sleep (activeservice->rev_interval);

	/* Now do our stuff. */
	for (current_backend = 0;
	     current_backend < activeservice->nbackend;
	     current_backend++) {

	    /* Only loop through unavailable back ends. */
	    if (servicereport->backendstate[current_backend].avail !=
		st_unavailable)
		continue;

	    /* Mark state as WAKING */		    
	    lock_reporter();
	    servicereport->backendstate[current_backend].avail = st_waking;
	    unlock_reporter();

	    /* Try to TCP connect */
	    sock = backend_connect();
	    close (sock);

	    /* Set state accordingly */
	    if (sock >= 0)
		warning ("Backend %s of service %s has woken up",
			 activeservice->backend[current_backend].name,
			 activeservice->name);
	    
	    lock_reporter();
	    servicereport->backendstate[current_backend].avail =
		sock >= 0 ? st_available : st_unavailable;
	    unlock_reporter();	   
	}
    }
}
